#!/bin/busybox sh

usage() {
    echo "Usage:"
    echo "  $0 [options] <target device file name> [ <image> ]"
    echo "Options:"
    echo "  -h"
    echo "     Show this help message."
    echo "  -s"
    echo "     Bytes in input, if not reading from file"
    echo "  -t"
    echo "     flash type: nand, nor, ubi, emmc"
    echo
    echo "  image"
    echo "     image to be stored on device, if emtpy stdin is assumed"
    echo
    echo " Exit codes:"
    echo "   0:  Success."
    echo "   1:  Invalid arguments."
}

FLASH_TYPE=
IMAGE_BYTES=
ERASE=

while getopts "hs:t:" option; do
    case $option in
        h)
            usage
            exit 0
            ;;
        s)
            IMAGE_BYTES="$OPTARG"
            ;;
        t)
            FLASH_TYPE="$OPTARG"
            ;;
        \?)
            usage
            exit 1
            ;;
    esac
done

shift $((OPTIND - 1))

DEVICE="$1"
IMAGE="$2"

case "$FLASH_TYPE" in
    nand|nor)
        # get the char device
        DEVICE=$(echo $DEVICE | sed 's/mtdblock/mtd/')
        ;;
    emmc|ubi)
        ;;
    *)
        echo "ERROR unknown flash type $FLASH_TYPE"
        ;;
esac

if [ -z "$DEVICE" ] || [ ! -w "$DEVICE" ] ; then
    echo "ERROR missing device node $DEVICE"
    usage
    exit 1
fi

if [ -z "$IMAGE" ]; then
    # reading IMAGE from stdin
    if [ -z "$IMAGE_BYTES" ]; then
        echo "ERROR size of image needed when reading from stdin"
        exit 1
    fi
else
    if [ ! -r "$IMAGE" ]; then
        echo "ERROR file not readable '$IMAGE'"
        exit 2
    fi
    # NOTE: overwrites IMAGE_BYTES if set
    IMAGE_BYTES=$(stat -c "%s" $IMAGE)
fi

mtd_size_bytes()
{
    local MTD=$(basename "$DEVICE")
    local HEX=$(awk "/${MTD}:/ {print \$2;}" "$RESCUE_ROOT_DIR"/proc/mtd)
    echo $((0x$HEX + 0))
}

blockdev_size_bytes()
{
    "$RESCUE_ROOT_DIR"/sbin/blockdev --getsize64 "$DEVICE"
}

DEVICE_BYTES=
case $FLASH_TYPE in
    nand|nor)
        DEVICE_BYTES=$(mtd_size_bytes)
        ;;
    emmc)
        DEVICE_BYTES=$(blockdev_size_bytes)
        ;;
    ubi)
        # ubi tool checks size
        ;;
esac

if [ $DEVICE_BYTES ] && [ $IMAGE_BYTES -gt $DEVICE_BYTES ]; then
    echo "ERROR target device too small, $IMAGE_BYTES > $DEVICE_BYTES"
    exit 4
fi

write_mtd()
{
    "$RESCUE_ROOT_DIR"/usr/sbin/flash_erase "$DEVICE" 0 0
    if [ -n "$IMAGE" ]; then
        exec "$RESCUE_ROOT_DIR"/usr/sbin/nandwrite "$DEVICE" "$IMAGE"
    else
        exec "$RESCUE_ROOT_DIR"/usr/sbin/nandwrite "$DEVICE" -
    fi
}

write_blockdev()
{
    if [ -n "$IMAGE" ]; then
        exec "$RESCUE_ROOT_DIR"/bin/dd if="$IMAGE" of="$DEVICE"
    else
        exec "$RESCUE_ROOT_DIR"/bin/dd of="$DEVICE"
    fi
}

write_ubi()
{
    if [ -z "$IMAGE" ]; then
        exec "$RESCUE_ROOT_DIR"/usr/sbin/ubiupdatevol "$DEVICE"\
            -s $IMAGE_BYTES -
    else
        exec "$RESCUE_ROOT_DIR"/usr/sbin/ubiupdatevol "$DEVICE"\
            "$IMAGE"
    fi
}

case $FLASH_TYPE in
    nand|nor)
        write_mtd
        ;;
    emmc)
        write_blockdev
        ;;
    ubi)
        write_ubi
        ;;
esac

# not reached
exit 0
